<?php

?>
<?php

/**
 * This class is use to communicate with Netsuite via it's RESTful API
 */
class CloudCoders_NetSuite_Helper_Sender extends Mage_Core_Helper_Abstract
{

    const DEBUG_TO_SESSION = false;
    const DEBUG_TO_LOG = false;

    private $NetSuite_Customer_Deployment = "1"; // The Customer RESTlet script is deployed
    private $netsuiteOrderDeployment = "1"; // The sales order RETlet script is deployed
    private $errorMessage = "";

    /**
     * Appends an error/debug message to the log/session
     *
     * @param mix $pMessage
     */
    private function setDebugMessage($pMessage)
    {
        if (self::DEBUG_TO_SESSION) {
            Mage::getSingleton('core/session')->addNotice($pMessage, null, "cloudcoders.netsuite.log");
        }
        if (self::DEBUG_TO_LOG) {
            Mage::log($pMessage, null, "cloudcoders.netsuite.log");
        }
    }

    /**
     * Returns the process error message
     *
     * @return string
     */
    public function getErrorMessage()
    {
        return $this->errorMessage;
    }

    public function sendCustomerToNetsuite($pCustomerId, $debug = false)
    {

        $this->setDebugMessage('sendCustomerToNetsuite $pCustomerId=' . $pCustomerId);
        $netsuiteCustomerScript = Mage::getStoreConfig('system/netsuite/netsuite_customer_script');
        $Customer = Mage::helper('netsuite/factory')->newCustomer($pCustomerId);
        if ($debug) {
            return $Customer->toArray();
        }

        $jsonData = $Customer->toJson();

        $curl = $this->prepareNesuiteCurl($netsuiteCustomerScript, $this->NetSuite_Customer_Deployment, $jsonData);

        $hadSuccess = false;
        $warningMessage = "";
        $errorMessage = "";

        try {
            $retValJson = curl_exec($curl);
            $this->setDebugMessage('$retValJson=' . $retValJson);

            $http_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            $retVal = json_decode($retValJson, true);


            if ($http_status != '200') {
                $errorMessage = 'HTTP error: ' . $retValJson;
                $this->mErrorMessage = $errorMessage;
            } else if (isset($retVal["ErrorMessage"]) && $retVal["ErrorMessage"] != "") {
                $errMessage = $retVal["ErrorMessage"];
                $this->setDebugMessage('$errMessage=' . $errMessage);

                $errorMessage = 'NetSuite error: ' . $errMessage;
                $this->mErrorMessage = $errorMessage;
            } else if (isset($retVal["WarningMessage"]) && $retVal["WarningMessage"] != "") {
                $hadSuccess = true;

                $warnMessage = $retVal["WarningMessage"];
                $this->setDebugMessage('$warnMessage=' . $warnMessage);

                $warningMessage = 'NetSuite warning: ' . $warnMessage;
                $this->mWarningMessage = $warningMessage;
            } else {
                $hadSuccess = true;
            }
        } catch (Exception $e) {
            $errorMessage = 'Error: ' . $e->getMessage();
            $this->mErrorMessage = $errorMessage;
            $this->setDebugMessage('mErrorMessage=' . $errorMessage);
        }

        $logEntry = Mage::getModel('netsuite/log');
        if ($Customer->getMagentoCustomer()) {
            $logEntry->setTypeId($Customer->getMagentoCustomer()->getEntityTypeId());
        }
        $logEntry->setEntityId($pCustomerId);
        $logEntry->setType("CUSTOMER");
        $logEntry->setActualMessage($jsonData);
        $logEntry->setSuccessFlag($hadSuccess);
        $logEntry->setWarningMessage($warningMessage);
        $logEntry->setErrorMessage($errorMessage);
        $logEntry->save();

        return array(
            'is_success' => $hadSuccess,
            'result' => $retVal,
        );
    }

    /**
     * Sends a Magento sales order to Netsuite
     *
     * @param int $pOrderId
     * @param boolean $debug Causes the method to return the processed sales order
     * @return array
     */
    public function sendOrderToNetsuite($pOrderId, $debug = false)
    {
        $netsuiteOrderScript = Mage::getStoreConfig('system/netsuite/netsuite_order_script');
        $Order = Mage::helper('netsuite/factory')->newOrder($pOrderId);
        $isSuccess = false;

        if ($debug) {
            return $Order->toArray();
        }

        try {
            $OrderData = $Order->toArray();
            $automated = Mage::getStoreConfig('system/netsuite/netsuite_automate_salesorder');
            if($automated){
                $OrderData['automated'] = true;
            }else{
                $OrderData['automated'] = false;
            }
            $Curl = $this->prepareNesuiteCurl($netsuiteOrderScript, $this->netsuiteOrderDeployment, json_encode($OrderData));
            $netsuiteResponse = json_decode(curl_exec($Curl), true);

            $http_status = curl_getinfo($Curl, CURLINFO_HTTP_CODE);
            if (
                    $http_status == '200' &&
                    isset($netsuiteResponse['salesorder_record']) &&
                    $netsuiteResponse['salesorder_record']) {
                    $netsuiteResponse['salesorder_record'] = json_decode($netsuiteResponse['salesorder_record'], true);
                if (is_array($netsuiteResponse['salesorder_record']) && $netsuiteResponse['salesorder_record']) {
                    Mage::app()->getStore()->setConfig('cloudcoders_disable_order_event', true);
                    $Order->updateOrderComment('Order synced to Netsuite successfully. Netsuite Id: ' . $netsuiteResponse['salesorder_record']['id'] . '  Transaction Id: ' . $netsuiteResponse['salesorder_record']['tranid']);
                }
                $isSuccess = true;
            } else {
                $Order->log($netsuiteResponse);
            }
            curl_close($Curl);
        } catch (Exception $e) {
            $this->errorMessage = 'Error: ' . $e->getMessage();
            $Order->logTofile($this->errorMessage);
            return $this->errorMessage;
        }

        $logEntry = Mage::getModel('netsuite/log');
        $logEntry->setTypeId(Mage::getModel('eav/entity')->setType('order')->getEntityType()->getEntityTypeId());
        $logEntry->setEntityId($pOrderId);
        $logEntry->setType("ORDER");
        $logEntry->setActualMessage($Order->toJson());
        $successMsg = (isset($netsuiteResponse['success']))? $netsuiteResponse['success']: false;
        $logEntry->setSuccessFlag($successMsg);
        $warningMsg = (isset($netsuiteOrderScript['warning_message']))?$netsuiteOrderScript['warning_message']: ' ';
        $logEntry->setWarningMessage($warningMsg);
        $errorMsg = (isset($netsuiteResponse['error_message']))?$netsuiteResponse['error_message']: ' ';
        $logEntry->setErrorMessage($errorMsg);
        $logEntry->save();

        return array(
            'is_success' => $isSuccess,
            'result' => $netsuiteResponse
        );
    }

    /**
     * Returns a prepared CURL object
     *
     * @param int $scriptId
     * @param array $data
     * @param int $scriptDeploy
     * @return array
     */
    public function curlGet($scriptId, $data = array(), $scriptDeploy = 1)
    {
        $Curl = $this->prepareNesuiteCurl($scriptId, $scriptDeploy, $data);
        return array(
            'Curl' => $Curl,
            'result' => curl_error($Curl),
        );
    }

    /**
     * Prepares a CURL object using the provided parameters
     *
     * @param int $scriptId
     * @param int $isDeployed
     * @param array $pData
     * @return mix [Object | false]
     */
    private function prepareNesuiteCurl($scriptId, $isDeployed, $pData)
    {
        $NetSuite_Account = Mage::getStoreConfig('system/netsuite/netsuite_account');
        $NetSuite_User = Mage::getStoreConfig('system/netsuite/netsuite_user');
        $NetSuite_Password = Mage::getStoreConfig('system/netsuite/netsuite_password');
        $NetSuite_Role = Mage::getStoreConfig('system/netsuite/netsuite_role');

        $url = $this->getScriptUrl($scriptId, $isDeployed);
        if (!$url) {
            return false;
        }


        $authString = "NLAuth nlauth_account=" . trim($NetSuite_Account) . ",nlauth_email=" . trim($NetSuite_User) . ",nlauth_signature=" . $NetSuite_Password . ",nlauth_role=" . trim($NetSuite_Role);


        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $pData);
        curl_setopt($curl, CURLINFO_HEADER_OUT, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array("Authorization: " . $authString, "Content-Type: application/json"));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($curl, CURLOPT_CAINFO, dirname(__FILE__) . "/../certs/BuiltinObjectTokenGoDaddyClass2CA.crt");

        // Additional       
        // Don't wait for response
        $closeConneciton = Mage::getStoreConfig('system/netsuite/netsuite_automate_salesorder');
        if ($closeConneciton) {
            curl_setopt($curl, CURLOPT_FRESH_CONNECT, true);
            if (defined('CURLOPT_TIMEOUT_MS')) {
                curl_setopt($curl, CURLOPT_TIMEOUT_MS, 1000); // Less than one second is too quick
            } else {
                curl_setopt($curl, CURLOPT_TIMEOUT, 1);
            }
        }
        return $curl;
    }

    /**
     * Returns the full REST script url
     *
     * @param int $scriptId
     * @param boolean $isDeployed
     * @return mix string | false
     */
    public function getScriptUrl($scriptId, $isDeployed = true)
    {
        $baseUrl = $this->getBaseUrl();
        if ($baseUrl) {
            return $url = $baseUrl . "/app/site/hosting/restlet.nl?script=" . $scriptId . "&deploy=" . $isDeployed;
        }
        return false;
    }

    /**
     * Returns netsuite REST script base url
     *
     * @return mix string | boolean
     */
    public function getBaseUrl()
    {
        $NetSuite_Account = Mage::getStoreConfig('system/netsuite/netsuite_account');
        $NetSuite_User = Mage::getStoreConfig('system/netsuite/netsuite_user');
        $NetSuite_Password = Mage::getStoreConfig('system/netsuite/netsuite_password');
        $NetSuite_Role = Mage::getStoreConfig('system/netsuite/netsuite_role');

        $NetSuite_Url = Mage::getStoreConfig('system/netsuite/netsuite_url');

        if ($NetSuite_Url == null || $NetSuite_Url == "") {
            $NetSuite_Url = "https://rest.netsuite.com";

            //Establish the correct URL via REST Role Discovery
            $discoveryUrl = "https://rest.netsuite.com/rest/roles";
            $authString = "NLAuth nlauth_email=" . $NetSuite_User . ",nlauth_signature=" . $NetSuite_Password;

            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL, $discoveryUrl);
            curl_setopt($curl, CURLINFO_HEADER_OUT, true);
            curl_setopt($curl, CURLOPT_HTTPHEADER, array("Authorization: " . $authString, "Accept: */*"));
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
            curl_setopt($curl, CURLOPT_CAINFO, dirname(__FILE__) . "/../certs/BuiltinObjectTokenGoDaddyClass2CA.crt");

            try {
                $retValJson = curl_exec($curl);



                $http_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);

                curl_close($curl);

                $this->setDebugMessage('$http_status=' . $http_status);

                $retVal = json_decode($retValJson, true);

                if ($http_status == '200') {
                    foreach ($retVal as $entry) {
                        if ($entry['account']['internalId'] == $NetSuite_Account && $entry['role']['internalId'] == $NetSuite_Role) {
                            $NetSuite_Url = $entry['dataCenterURLs']['restDomain'];
                            Mage::getModel('core/config')->saveConfig('system/netsuite/netsuite_url', $NetSuite_Url);

                            break;
                        }
                    }
                }
            } catch (Exception $e) {
                $this->setDebugMessage('curl error =' . $e->getMessage());
                return false;
            }
        }

        return $NetSuite_Url;
    }

}