<?php

?>
<?php

/**
 * This class processes queued sale order and customer data
 */
class CloudCoders_NetSuite_Model_Queue extends Mage_Core_Model_Abstract
{

    const DEBUG_TO_SESSION = false;
    const DEBUG_TO_LOG = false;
    const IN_QUEUE = 1;
    const PROCESSING = 2;
    const COMPLETE = 3;

    protected function _construct()
    {
        $this->_init('netsuite/queue');
    }

    /**
     * Processes queued sales order and customer
     *
     * This is the function that is call by magento
     * cron job manager
     *
     * @param bool $ignoreErrorFlag
     * @return null
     */
    public function process_queue($ignoreErrorFlag = false)
    {
        $this->debug_message('process_queue');

        $isSyncingOff = Mage::getStoreConfig('system/netsuite/netsuite_disable_sync');
        $ignoreErrorFlag = Mage::getStoreConfig('system/netsuite/netsuite_resync_failed_sync');
        $orderSyncStartDate = Mage::getStoreConfig('system/netsuite/netsuite_order_sync_start_date');
        if ($orderSyncStartDate) {
            $orderSyncStartDate = strtotime($orderSyncStartDate);
        }

        // Syncing off
        if ($isSyncingOff) {
            return;
        }
        $customerTypeId = Mage::getModel('eav/entity')->setType('customer')->getEntityType()->getEntityTypeId();
        $this->debug_message('$customerTypeId=' . $customerTypeId);

        $orderTypeId = Mage::getModel('eav/entity')->setType('order')->getEntityType()->getEntityTypeId();

        $this->debug_message('$orderTypeId=' . $orderTypeId);

        $entries = Mage::getModel('netsuite/queue')->getCollection();

        if (!$ignoreErrorFlag) {
            $entries->addFieldToFilter('error_flag', 0);
        }
        $entries->setOrder('timestamp', 'ASC');

        $this->debug_message('$entries->count()=' . $entries->count());
        if ($entries->count() != 0) {
            $helper = Mage::helper('netsuite/sender');

            foreach ($entries as $queueEntry) {
                $History = Mage::getModel('netsuite/history');
                $this->debug_message($queueEntry->getQueueId() . " " . $queueEntry->getEntityId());

                $typeId = $queueEntry->getTypeId();
                $entityId = $queueEntry->getEntityId();



                $sendResult = false;
                $noSenderFound = false;
                if ($typeId == $customerTypeId) {
                    $currentCustomerTime = strtotime($queueEntry->getTimestamp());
                    if ($currentCustomerTime >= $orderSyncStartDate) {
                        $result = $helper->sendCustomerToNetsuite($entityId);
                        $sendResult = $result['result']['success'];

                        // add History
                        $History->setEntityType('customer');
                        $History->setMagentoId($entityId);
                        $History->setSyncDirection('Magento to NetSuite');
                        if ($result['result']['success']) {
                            $customerId = (isset($result['customer_id']))?$result['customer_id']:' ';
                            $History->setNetsuiteId($customerId);
                        }

                        $History->setSyncData('Magento customer Id: ' . $entityId);
                        $History->setSyncMessage(json_encode($result));
                        $History->save();
                    } else {
                        $sendResult = true;
                    }
                } else if ($typeId == $orderTypeId) {

                    $Order = Mage::helper('netsuite/factory')->newOrder($entityId)->toArray();
                    // Order does not exist
                    if (empty($Order)) {
                        $sendResult = true;
                    } else {

                        $currentOrderTime = strtotime($Order['created_at']);
                        if ($currentOrderTime >= $orderSyncStartDate) {
                            $result = $helper->sendOrderToNetsuite($entityId);

                            if (trim(strtolower($result['result']['warning_message'])) == "sales order already exist") {
                                $sendResult = true;
                            } else {
                                $sendResult = $result['result']['success'];

                                // add History
                                $History->setEntityType('sales order');
                                $History->setMagentoId($entityId);
                                $History->setSyncDirection('Magento to NetSuite');
                                if ($result['result']['success']) {
                                    $History->setNetsuiteId($result['result']['saleorder_record']['itemid']);
                                }

                                $History->setSyncData('Magento sales order Id: ' . $entityId);
                                $History->setSyncMessage(json_encode($result));
                                $History->save();
                            }
                        } else {
                            $sendResult = true;
                        }
                    }
                } else {
                    $noSenderFound = true;
                }

                if ($sendResult) {
                    $queueEntry->delete();
                } else {
                    $queueEntry->setErrorFlag(true);

                    if ($noSenderFound) {
                        $queueEntry->setErrorMessage("Don't know how to send entity type " . $entityId);
                    } else {
                        $queueEntry->setErrorMessage($helper->getErrorMessage());
                    }
                    $queueEntry->save();
                }

                $this->debug_message('In queue, error message is ' . $helper->getErrorMessage());
            }
        }
    }

    /**
     * Processes all queued jobs from netsuite
     *
     */
    public function processNetsuiteToMagentoQueue()
    {
        set_time_limit(0);

        $QueueModel = Mage::getModel('netsuite/simpleproduct');
        $CollectionObj = $QueueModel->getCollection();

        $isSyncingOff = Mage::getStoreConfig('system/netsuite/netsuite_disable_sync');
        // Syncing off
        if ($isSyncingOff) {
            return false;
        }

        foreach ($CollectionObj as $curJob) {

            $History = Mage::getModel('netsuite/history');
            $jobData = $curJob->getData();

            $jobData['job_item'] = json_decode($jobData['job_item'], true);

            // Delete completed job
            if ($jobData['job_status'] == self::COMPLETE || (!$jobData['job_item']['end_point'])) {
                $curJob->delete();
                continue;
            }

            $History->setEntityType($jobData['job_item']['end_point']);

            // Do all queued jobs related to the product end point
            $this->processProductEndPoint($jobData, $curJob, $History);

            // Do all queued jobs relating to customer end point
            $this->processCustomerEndPoint($jobData, $curJob, $History);

            // Do all queued jobs relating to salesorder end point
            $this->processSalesOrderEndPoint($jobData, $curJob, $History);
            
            // Do all queued jobs relating to bulkimport end point
            $this->processBulkImportEndPoint($jobData, $curJob, $History);
        }

       // $process = Mage::getModel('index/process')->load(6);
        $process = Mage::getSingleton('index/indexer')->getProcessByCode('catalog_category_product');
        $process->reindexAll();

        // House cleaning
        $uploadFolderPath = Mage::getBaseDir('media') . DIRECTORY_SEPARATOR . Mage::getStoreConfig('system/netsuite/netsuite_media_temp_folder');
        $files = glob($uploadFolderPath . '/*.*');
        foreach ($files as $file) {
          
           $isImage = @getimagesize($file);
            if (is_file($file) && $isImage) {
                @unlink($file); // only delete images
            }
        }
    }

    /**
     * Add an item to the queue(module table)
     *
     * Item's data that are sent from netsuite are
     * saved and at a set time the data is processed,
     *
     * @param string $method The RESTful CRUD
     * @param array $data
     * @return array success status and message
     */
    public function addJobFromNetsuite($method, $data)
    {

        $newJob = Mage::getModel('netsuite/simpleproduct');
        $data['job_added_time'] = time();
        $newJob->setData('job_status', self::IN_QUEUE);
        $newJob->setData('job_item', json_encode($data));
        $newJob->setData('job_task', $method);
        $newJob->save();

        $itemNetsuiteId = (isset($data['item_netsuite_id'])) ? $data['item_netsuite_id'] : '';
        $result = array(
            'success' => true,
            'message' => 'Job for netsuite item: ' . $itemNetsuiteId . ' queued successfully',
        );
        return $result;
    }

    /**
     * Stores debug message either to session or database
     *
     * @param string $pMessage
     */
    private function debug_message($pMessage)
    {
        if (self::DEBUG_TO_SESSION) {
            Mage::getSingleton('core/session')->addNotice($pMessage);
        }
        if (self::DEBUG_TO_LOG) {
            Mage::log($pMessage);
        }
    }

    /**
     * Processes Netsuite item to magento product queued jobs
     * 
     * @param array $jobData 
     * @param array $curJob
     * @param object $History
     * @return boolean
     */
    private function processProductEndPoint($jobData, $curJob, $History)
    {
        // Do all queued jobs related to the product end point
        if ($jobData['job_item']['end_point'] == 'product') {

            unset($jobData['job_item']['end_point']);
            $SimpleProductModel = Mage::getModel('netsuite/simpleproduct');
            $SimpleProductModel->setItemNetsuiteData($jobData['job_item']);
            //History
            $History->setNetsuiteId($jobData['job_item']['item_netsuite_id']);

            switch ($curJob['job_task']) {
                case 'POST':
                    $curJob->setData('job_status', self::PROCESSING);
                    $curJob->save();

                    $result = $SimpleProductModel->addProduct($jobData['job_item']);
                    $curJob->setData('job_status', self::COMPLETE);
                    $curJob->save();

                    // History
                    $History->setMagentoId($result['result']['magento_id']);
                    $History->setSyncMessage($result['message']);

                    break;
                case 'DELETE':

                    $curJob->setData('job_status', self::PROCESSING);
                    $curJob->save();
                    $SimpleProductModel->deleteProduct($jobData['job_item'], true);
                    $curJob->setData('job_status', self::COMPLETE);
                    $curJob->save();

                    // History
                    $History->setMagentoId($jobData['job_item']['item_netsuite_id']);
                    $History->setSyncMessage('NetSuite item was deleted successfully');

                    break;
                default:
                    break;
            }

            // History
            $History->setSyncData(json_encode($jobData));
            // Add History Record
            $History->setSyncDirection('NetSuite to Magento');
            $History->save();
            return true;
        }
        return false;
    }

    /**
     * Processes Netsuite customer to magento customer queued jobs
     * 
     * @param array $jobData
     * @param array $curJob
     * @param object $History
     * @return boolean
     */
    private function processCustomerEndPoint($jobData, $curJob, $History)
    {
        if ((isset($jobData['job_item'])) && isset($jobData['job_item']['end_point']) && $jobData['job_item']['end_point'] == 'customer') {
            $data = $jobData['job_item']['record'];


            // Additional data fields
            $data['store_id'] = $jobData['job_item']['store_id'];
            $data['customer_group'] = $jobData['job_item']['customer_group'];
            $data['customer_magento_id'] = $jobData['job_item']['customer_magento_id'];
            $data['group_price'] = $jobData['job_item']['group_price'];
            $data['item_price'] = $jobData['job_item']['item_price'];

            switch ($curJob['job_task']) {
                case 'POST':
                    $curJob->setData('job_status', self::PROCESSING);
                    $curJob->save();

                    $NestuiteCustomer = Mage::helper('netsuite/factory')->newNesuiteCustomer($data);
                    $NestuiteCustomer->saveCustomer();
                    $curJob->setData('job_status', self::COMPLETE);
                    $curJob->save();

                    //History
                    $History->setMagentoId($data['customer_magento_id']);
                    $History->setNetsuiteId($data['entityid']);

                    if ($data['customer_magento_id']) {
                        $History->setSyncMessage('NetSuite Customer was updated successfully');
                    } else {
                        $History->setSyncMessage('NetSuite Customer was added successfully');
                    }

                    break;
                default:
                    break;
            }

            // History
            $History->setSyncData(json_encode($data));
            // Add History Record
            $History->setSyncDirection('NetSuite to Magento');
            $History->save();
            return true;
        }
        return false;
    }

    /**
     * Processes Magento sales order comment update queued jobs
     * 
     * @param array $jobData
     * @param array $curJob
     * @param type $History
     * @return boolean
     */
    private function processSalesOrderEndPoint($jobData, $curJob, $History)
    {
        if ((isset($jobData['job_item'])) && isset($jobData['job_item']['end_point']) && $jobData['job_item']['end_point'] == 'salesorder') {

            switch ($curJob['job_task']) {
                case 'POST':
                    // Chanage queue status
                    $curJob->setData('job_status', self::PROCESSING);
                    $curJob->save();

                    // Update sales order comment
                    $jobData['job_item']['salesorder_record'] = json_decode($jobData['job_item']['salesorder_record'], true);
                    $data = $jobData['job_item'];

                    $orderId = $data['salesorder_record']['custbody_ccmi_order_id'];
                    $Order = Mage::helper('netsuite/factory')->newOrder($orderId);
                    if ($Order) {
                        Mage::app()->getStore()->setConfig('cloudcoders_disable_order_event', true);
                        $Order->updateOrderComment('Order synced to Netsuite successfully. Netsuite Id: ' . $data['salesorder_record']['id'] . '  Transaction Id: ' . $data['salesorder_record']['tranid']);
                    }
                    // Complete satus
                    $curJob->setData('job_status', self::COMPLETE);
                    $curJob->save();
                    break;
                default:
                    break;
            }

            // History
            $History->setSyncData(json_encode($jobData));
            // Add History Record
            $History->setSyncDirection('NetSuite to Magento');
            $History->save();
            return true;
        }
        return false;
    }

    /**
     * Processes Netsuite bulk items import queued jobs
     * 
     * @param array $jobData
     * @param array $curJob
     * @param object $History
     * @return boolean
     */
    private function processBulkImportEndPoint($jobData, $curJob, $History)
    {
        if ((isset($jobData['job_item'])) && isset($jobData['job_item']['end_point']) && $jobData['job_item']['end_point'] == 'bulkimport') {

            switch ($curJob['job_task']) {
                case 'POST':
                     // Chanage queue status
                    $curJob->setData('job_status', self::PROCESSING);
                    $curJob->save();
                    
                    // Do processing
                    $BulkImportModel = Mage::getModel('netsuite/bulkImport');
                    $BulkImportModel->createProduct($jobData['job_item']);
                    
                    // Complete satus
                    $curJob->setData('job_status', self::COMPLETE);
                    $curJob->save();
                    break;
            }
            
            // History
            $History->setSyncData(json_encode($jobData));
            // Add History Record
            $History->setSyncDirection('NetSuite to Magento');
            $History->save();
            return true;
        }
        return false;
    }

}